::This file was made by inoage GmbH, http://www.inoage.de/

::Usage: build-OSG-3.0-VC16-x64.bat [/code=<source code directory>] [/build=<build directory>] [/deploy=<deploy directory>] [/logs=<logs directory>] [/qt=<Qt directory>] [/t:<target>]
::Builds OSG 3.0 (x64) for VC16

::Dependencies:
::CMake must be installed and included in the PATH environment variable

::Hints:
::The source code directory must not contain spaces in the path
::OSG 3.0 supports out-of-source builds

::References:
::http://www.openscenegraph.org/index.php/documentation/getting-started

@echo off
setlocal enabledelayedexpansion

::Set up the environment
set repoDir=..\..\..\..
set PATH=%repoDir%\sdk\utility;%PATH%
call "C:\Program Files (x86)\Microsoft Visual Studio\2019\Enterprise\VC\Auxiliary\Build\vcvarsall.bat" amd64 2> nul || ^
call "C:\Program Files (x86)\Microsoft Visual Studio\2019\Professional\VC\Auxiliary\Build\vcvarsall.bat" amd64 2> nul || (
echo The setup script for the Visual Compiler environment ^(VC16^) could not be found^^! & pause & exit /b 1)
echo.

::Parse the arguments
set defaultBaseDir=C:\SDK\OSG\OSG-3.0.1
set codeDir=%defaultBaseDir%\code
set buildDir=%defaultBaseDir%\build\VC16\x64
set deployDir=%defaultBaseDir%\deploy\VC16\x64
set logsDir=
set qtDir=C:\SDK\Qt\Qt-5.9.7\deploy\VC16\x64
set defaultTargets=:clean-up-build :clean-up-deploy :clean-up-logs :config :build :deploy
set targets=%defaultTargets%

:beginParseArguments
set p=%1
if _%p%_==__ goto :endParseArguments
if /i _%p%_==_/code_      (shift & set codeDir=%~f2) else ^
if /i _%p%_==_/build_     (shift & set buildDir=%~f2) else ^
if /i _%p%_==_/deploy_    (shift & set deployDir=%~f2) else ^
if /i _%p%_==_/logs_      (shift & set logsDir=%~f2) else ^
if /i _%p%_==_/qt_        (shift & set qtDir=%~f2) else ^
if /i _%p%_==_/t:clean-up-build_  (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up-deploy_ (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up-logs_   (set targets=%p:~2%) else ^
if /i _%p%_==_/t:build_           (set targets=%p:~2%) else ^
if /i _%p%_==_/t:deploy_          (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up_        (set targets=:clean-up-build :clean-up-deploy :clean-up-logs) else ^
if /i _%p%_==_/t:reconfig_        (set targets=:clean-up-build :clean-up-deploy :clean-up-logs :config) else ^
if /i _%p%_==_/t:rebuild_         (set targets=:clean-up-build :clean-up-deploy :clean-up-logs :config :build) else ^
if /i _%p%_==_/t:build-deploy_    (set targets=:build :deploy) else ^
if /i _%p%_==_/t:all_             (set targets=%defaultTargets%) else ^
if /i _%p%_==_/t:all-tidy_        (set targets=%defaultTargets% :clean-up-build) else ^
echo Skipping the invalid argument '%p%'...
shift
goto :beginParseArguments
:endParseArguments

if "%logsDir%"=="" set logsDir=%deployDir%\logs

echo Description: Builds OSG 3.0 ^(x64^) for VC16
echo Source code: '%codeDir%'
echo Build      : '%buildDir%'
echo Deploy     : '%deployDir%'
echo Logs       : '%logsDir%'
echo Targets    : '%targets%'
echo.
echo Qt         : '%qtDir%'
"%qtDir%\bin\qmake.exe" -v || (echo The executable 'qmake.exe' could not be found in the Qt binary directory^^! & pause & exit /b 1)
echo.
echo These settings have been deduced from command line arguments or set to the default values.
set /p c=Do you want to continue [yn]? & if _!c!_==__ set c=y
if /i not _%c%_==_y_ (echo Canceled. & exit /b 0)
echo.

::Call the targets
for %%I in (%targets%) do (call %%I || (pause & exit /b 1))
pause & exit /b 0

:clean-up-build
if not exist "%buildDir%" goto :eof
echo Cleaning up the build directory...
call rmPath.bat "%buildDir%"
echo.
goto :eof

:clean-up-deploy
if not exist "%deployDir%" goto :eof
echo Cleaning up the deploy directory...
call rmPath.bat "%deployDir%"
echo.
goto :eof

:clean-up-logs
if not exist "%logsDir%" goto :eof
echo Cleaning up the logs directory...
call rmPath.bat "%logsDir%"
echo.
goto :eof

:config
::Check for the source code directory
if not exist "%codeDir%" (echo The source code directory could not be found^^! & exit /b 1)
if not exist "%codeDir%\CMakeLists.txt" (echo The project 'CMakeLists.txt' could not be found in the source code directory^^! & exit /b 1)

echo Creating the build, deploy and logs directories...
mkdir "%buildDir%"
mkdir "%deployDir%"
mkdir "%logsDir%"
echo.

echo Configuring the flags...
set logFile=%logsDir%\configure.log
set logFileV=%logsDir%\configure_verbose.log
echo See '%logFile%'
echo and '%logFileV%' for details.
set configureFlags=

::Deploy out-of-source
set configureFlags=%configureFlags% -DCMAKE_INSTALL_PREFIX:PATH="%deployDir:\=/%"

::Target specification
set configureFlags=%configureFlags% -G "Visual Studio 16 2019" -A "x64"
set configureFlags=%configureFlags% -DCMAKE_CONFIGURATION_TYPES:STRING="Debug;Release;RelWithDebInfo"

::Multi-threaded compilation (via MSVC)
set configureFlags=%configureFlags% -DWIN32_USE_MP:BOOL="1"

::Enable address space layout randomization (/DYNAMICBASE)
set configureFlags=%configureFlags% -DWIN32_USE_DYNAMICBASE:BOOL="1"

::Provide the Qt package directory for CMake
set configureFlags=%configureFlags% -DQt5_DIR:PATH="%qtDir:\=/%/lib/cmake/Qt5"

::Do not use the third-party dependency collection
set configureFlags=%configureFlags% -DUSE_3RDPARTY_BIN:BOOL="0"

::Enable workaround for multi-threaded OpenGL driver occlussion query crash
set configureFlags=%configureFlags% -DOSG_FORCE_QUERY_RESULT_AVAILABLE_BEFORE_RETRIEVAL:BOOL="1"

::Configure out-of-source
cd /d "%buildDir%"
cmake.exe %configureFlags% -S "%codeDir%" -B "%buildDir%" > "%logFile%" 2> "%logFileV%"
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof

:build
echo Compiling...
set logFile=%logsDir%\compile.log
echo See '%logFile%' for details.

::Build out-of-source
cd /d "%buildDir%"

::Build the project ALL_BUILD once per configuration
::Options: https://docs.microsoft.com/en-us/visualstudio/msbuild/msbuild-command-line-reference?view=vs-2019
set innerCommand0=MSBuild.exe -t:build -p:configuration=Debug   -p:platform=x64 -nologo -v:minimal -flp:logfile="%logFile%";v=minimal        ".\ALL_BUILD.vcxproj"
set innerCommand1=MSBuild.exe -t:build -p:configuration=Release -p:platform=x64 -nologo -v:minimal -flp:logfile="%logFile%";v=minimal;append ".\ALL_BUILD.vcxproj"

::Options: /high ..... Schedule with high priority
::         /wait ..... Wait for the inner command to finish
start /high /wait %innerCommand0%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
start /high /wait %innerCommand1%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof

:deploy
echo Installing...
set logFile=%logsDir%\install.log
echo See '%logFile%' for details.

::Deploy out-of-source
cd /d "%buildDir%"

::Build the project INSTALL once per configuration
::Options: https://docs.microsoft.com/en-us/visualstudio/msbuild/msbuild-command-line-reference?view=vs-2019
set innerCommand0=MSBuild.exe -t:build -p:configuration=Debug   -p:platform=x64 -nologo -v:minimal -flp:logfile="%logFile%";v=minimal        ".\INSTALL.vcxproj"
set innerCommand1=MSBuild.exe -t:build -p:configuration=Release -p:platform=x64 -nologo -v:minimal -flp:logfile="%logFile%";v=minimal;append ".\INSTALL.vcxproj"

::Options: See above
start /high /wait %innerCommand0%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
start /high /wait %innerCommand1%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof
